﻿#region Using directives

using System;
using System.Collections.Generic;
using STL.Generic;
using Fn = STL.Generic.Functional;
using IntVector = STL.Generic.Vector<int>;
using IntIter = STL.Generic.IForwardIterator<int>;
using Result = STL.Generic.Tuple<int, int, int, double>;

#endregion

class Program
{
    static void Print(IntIter first, IntIter last)
    {
        Algorithm.ForEach<int>(first, last, 
            delegate(int n) { Console.Write("{0} ", n); });
        Console.WriteLine();
    }

    static void TestVector()
    {
        // キャパシティ10でVectorを作成
        IntVector iv = new IntVector(10);

        // 0～9まで値を入れる
        for (int i = 0; i < iv.Size; ++i)
            iv[i] = i;

        // 0～9が表示される
        Print(iv.Begin(), iv.End());

        // 9番目の要素が削除される
        iv.Erase(iv.Begin().Offset(8));
        // 8が消えている
        Print(iv.Begin(), iv.End());
        // 2番目から5番目(9-3番目を含まない位置)までの要素が削除される
        iv.Erase(iv.Begin().Offset(1), iv.End().Offset(-3));
        // 1～5が消えている
        Print(iv.Begin(), iv.End());
        // 先頭から2要素が削除される
        iv.Erase(iv.Begin(), iv.Begin().Offset(2));
        // 0,6が消えている
        Print(iv.Begin(), iv.End());
        // 全て削除
        iv.Clear();
        if (iv.Empty())
            Console.WriteLine("Vector is empty.");

        // ReverseIterator

        // 0～9まで値を入れる
        for (int i = 0; i < 10; ++i)
            iv.PushBack(i);

        // 9～0が表示される
        Print(iv.Rbegin(), iv.Rend());
    }

    static void TestFunctional()
    {
        List<int> list = new List<int>();
        for (int i = 0; i < 10; ++i)
            list.Add(i);

        // 小なり
        BinaryFunction<int, int, bool> less = Fn.Less<int>();
        // 右項を7に固定
        UnaryFunction<int, bool> less7 = Fn.Bind2nd<int, int, bool>(less, 7);
        // リストから7より小さい要素を取得
        List<int> li = list.FindAll(new Predicate<int>(less7));
        foreach (int n in li)
            Console.Write("{0} ", n);
        Console.WriteLine();
        // less7の否定
        UnaryFunction<int, bool> not_less7 = Fn.Not1<int>(less7);
        // リストから７以上の要素を取得
        li = list.FindAll(new Predicate<int>(not_less7));
        foreach (int n in li)
            Console.Write("{0} ", n);
        Console.WriteLine();
    }

    // 型なしタプル
    static Tuple Calc4_1(int x, int y)
    {
        return Tuple.Make(x + y, x - y, x * y, (double)x / y);
    }

    // 型ありタプル
    static Result Calc4_2(int x, int y)
    {
        return Tuple.Make<Result>(x + y, x - y, x * y, (double)x / y);
    }

    static void TestTuple()
    {
        int add, sub, mul;
        double div;

        add = sub = mul = 0;
        div = 0.0;

        // 型なしタプルはインデクサでアクセス
        Tuple t1 = Calc4_1(3, 5);
        for (int i = 0; i < t1.Length; ++i)
            Console.Write("{0} ", t1[i]);
        Console.WriteLine();

        // 型ありタプルはフィールドを持つ
        Result t2 = Calc4_2(3, 5);
        Console.WriteLine("{0} {1} {2} {3}", t2._0, t2._1, t2._2, t2._3);

        // 型ありタプルへキャスト
        Result t3 = t1 as Result;

        // タプルから値を取得
        t3.Tie(ref add, ref sub, ref mul, ref div);
        Console.WriteLine("{0} {1} {2} {3}", add, sub, mul, div);
    }

    static void TestAlgorithm()
    {
        Vector<int> vi = new Vector<int>(10);
        // 全ての要素を10で埋める
        Algorithm.Fill<int>(vi.Begin(), vi.End(), 10);
        Print(vi.Begin(), vi.End());
        Random r = new Random();
        // 全ての要素に乱数を設定
        Algorithm.Generate<int>(vi.Begin(), vi.End(), Fn.Bind<int, int>(r.Next, 99));
        Print(vi.Begin(), vi.End());
    }

    static void Main(string[] args)
    {        
        Console.WriteLine("*** TestVector ***");
        TestVector();
        Console.WriteLine("*** TestFunctional ***");
        TestFunctional();
        Console.WriteLine("*** TestTuple ***");
        TestTuple();
        Console.WriteLine("*** TestAlgorithm ***");
        TestAlgorithm();

        Console.ReadLine();
    }
}
