﻿#region Using directives

using System;
using System.Collections.Generic;

#endregion

namespace STL.Generic
{
    // 算術演算インタフェース
    public interface IArithmetic<T>
        where T : IComparable, IFormattable, IConvertible, IComparable<T>
    {
        T Add(T x, T y);
        T Subtract(T x, T y);
        T Multiply(T x, T y);
        T Divide(T x, T y);
        T Modulus(T x, T y);
        T Negate(T x);
        T Zero { get;}
        T Unit { get;}
        T Value(int x);
        bool IsZero(T x);
        bool IsUnit(T x);
    }

    // デフォルトの実装
    public struct Arithmetic :
        IArithmetic<byte>, IArithmetic<sbyte>,
        IArithmetic<short>, IArithmetic<ushort>,
        IArithmetic<int>, IArithmetic<uint>,
        IArithmetic<long>, IArithmetic<ulong>,
        IArithmetic<float>, IArithmetic<double>
    {
        #region byte
        public byte Add(byte x, byte y) { return (byte)(x + y); }
        public byte Subtract(byte x, byte y) { return (byte)(x - y); }
        public byte Multiply(byte x, byte y) { return (byte)(x * y); }
        public byte Divide(byte x, byte y) { return (byte)(x / y); }
        public byte Modulus(byte x, byte y) { return (byte)(x % y); }
        public byte Negate(byte x) { throw new NotSupportedException(); }
        byte IArithmetic<byte>.Zero { get { return 0; } }
        byte IArithmetic<byte>.Unit { get { return 1; } }
        byte IArithmetic<byte>.Value(int x) { return (byte)x; }
        public bool IsZero(byte x) { return x == 0; }
        public bool IsUnit(byte x) { return x == 1; }
        public bool Equals(byte x, byte y) { return x == y; }
        public int GetHashCode(byte x) { return x.GetHashCode(); }
        #endregion

        #region sbyte
        public sbyte Add(sbyte x, sbyte y) { return (sbyte)(x + y); }
        public sbyte Subtract(sbyte x, sbyte y) { return (sbyte)(x - y); }
        public sbyte Multiply(sbyte x, sbyte y) { return (sbyte)(x * y); }
        public sbyte Divide(sbyte x, sbyte y) { return (sbyte)(x / y); }
        public sbyte Modulus(sbyte x, sbyte y) { return (sbyte)(x % y); }
        public sbyte Negate(sbyte x) { return (sbyte)-x; }
        sbyte IArithmetic<sbyte>.Zero { get { return 0; } }
        sbyte IArithmetic<sbyte>.Unit { get { return 1; } }
        sbyte IArithmetic<sbyte>.Value(int x) { return (sbyte)x; }
        public bool IsZero(sbyte x) { return x == 0; }
        public bool IsUnit(sbyte x) { return x == 1; }
        public bool Equals(sbyte x, sbyte y) { return x == y; }
        public int GetHashCode(sbyte x) { return x.GetHashCode(); }
        #endregion

        #region short
        public short Add(short x, short y) { return (short)(x + y); }
        public short Subtract(short x, short y) { return (short)(x - y); }
        public short Multiply(short x, short y) { return (short)(x * y); }
        public short Divide(short x, short y) { return (short)(x / y); }
        public short Modulus(short x, short y) { return (short)(x % y); }
        public short Negate(short x) { return (short)-x; }
        short IArithmetic<short>.Zero { get { return 0; } }
        short IArithmetic<short>.Unit { get { return 1; } }
        short IArithmetic<short>.Value(int x) { return (short)x; }
        public bool IsZero(short x) { return x == 0; }
        public bool IsUnit(short x) { return x == 1; }
        public bool Equals(short x, short y) { return x == y; }
        public int GetHashCode(short x) { return x.GetHashCode(); }
        #endregion

        #region ushort
        public ushort Add(ushort x, ushort y) { return (ushort)(x + y); }
        public ushort Subtract(ushort x, ushort y) { return (ushort)(x - y); }
        public ushort Multiply(ushort x, ushort y) { return (ushort)(x * y); }
        public ushort Divide(ushort x, ushort y) { return (ushort)(x / y); }
        public ushort Modulus(ushort x, ushort y) { return (ushort)(x % y); }
        public ushort Negate(ushort x) { throw new NotSupportedException(); }
        ushort IArithmetic<ushort>.Zero { get { return 0; } }
        ushort IArithmetic<ushort>.Unit { get { return 1; } }
        ushort IArithmetic<ushort>.Value(int x) { return (ushort)x; }
        public bool IsZero(ushort x) { return x == 0; }
        public bool IsUnit(ushort x) { return x == 1; }
        public bool Equals(ushort x, ushort y) { return x == y; }
        public int GetHashCode(ushort x) { return x.GetHashCode(); }
        #endregion

        #region int
        public int Add(int x, int y) { return x + y; }
        public int Subtract(int x, int y) { return x - y; }
        public int Multiply(int x, int y) { return x * y; }
        public int Divide(int x, int y) { return x / y; }
        public int Modulus(int x, int y) { return x % y; }
        public int Negate(int x) { return -x; }
        int IArithmetic<int>.Zero { get { return 0; } }
        int IArithmetic<int>.Unit { get { return 1; } }
        int IArithmetic<int>.Value(int x) { return x; }
        public bool IsZero(int x) { return x == 0; }
        public bool IsUnit(int x) { return x == 1; }
        public bool Equals(int x, int y) { return x == y; }
        public int GetHashCode(int x) { return x.GetHashCode(); }
        #endregion

        #region uint
        public uint Add(uint x, uint y) { return x + y; }
        public uint Subtract(uint x, uint y) { return x - y; }
        public uint Multiply(uint x, uint y) { return x * y; }
        public uint Divide(uint x, uint y) { return x / y; }
        public uint Modulus(uint x, uint y) { return x % y; }
        public uint Negate(uint x) { throw new NotSupportedException(); }
        uint IArithmetic<uint>.Zero { get { return 0; } }
        uint IArithmetic<uint>.Unit { get { return 1; } }
        uint IArithmetic<uint>.Value(int x) { return (uint)x; }
        public bool IsZero(uint x) { return x == 0; }
        public bool IsUnit(uint x) { return x == 1; }
        public bool Equals(uint x, uint y) { return x == y; }
        public int GetHashCode(uint x) { return x.GetHashCode(); }
        #endregion

        #region long
        public long Add(long x, long y) { return x + y; }
        public long Subtract(long x, long y) { return x - y; }
        public long Multiply(long x, long y) { return x * y; }
        public long Divide(long x, long y) { return x / y; }
        public long Modulus(long x, long y) { return x % y; }
        public long Negate(long x) { return -x; }
        long IArithmetic<long>.Zero { get { return 0; } }
        long IArithmetic<long>.Unit { get { return 1; } }
        long IArithmetic<long>.Value(int x) { return (long)x; }
        public bool IsZero(long x) { return x == 0; }
        public bool IsUnit(long x) { return x == 1; }
        public bool Equals(long x, long y) { return x == y; }
        public int GetHashCode(long x) { return x.GetHashCode(); }
        #endregion

        #region ulong
        public ulong Add(ulong x, ulong y) { return x + y; }
        public ulong Subtract(ulong x, ulong y) { return x - y; }
        public ulong Multiply(ulong x, ulong y) { return x * y; }
        public ulong Divide(ulong x, ulong y) { return x / y; }
        public ulong Modulus(ulong x, ulong y) { return x % y; }
        public ulong Negate(ulong x) { throw new NotSupportedException(); }
        ulong IArithmetic<ulong>.Zero { get { return 0; } }
        ulong IArithmetic<ulong>.Unit { get { return 1; } }
        ulong IArithmetic<ulong>.Value(int x) { return (ulong)x; }
        public bool IsZero(ulong x) { return x == 0; }
        public bool IsUnit(ulong x) { return x == 1; }
        public bool Equals(ulong x, ulong y) { return x == y; }
        public int GetHashCode(ulong x) { return x.GetHashCode(); }
        #endregion

        #region float
        public float Add(float x, float y) { return x + y; }
        public float Subtract(float x, float y) { return x - y; }
        public float Multiply(float x, float y) { return x * y; }
        public float Divide(float x, float y) { return x / y; }
        public float Modulus(float x, float y) { return x % y; }
        public float Negate(float x) { return -x; }
        float IArithmetic<float>.Zero { get { return 0; } }
        float IArithmetic<float>.Unit { get { return 1; } }
        float IArithmetic<float>.Value(int x) { return (float)x; }
        public bool IsZero(float x) { return x == 0; }
        public bool IsUnit(float x) { return x == 1; }
        public bool Equals(float x, float y) { return x == y; }
        public int GetHashCode(float x) { return x.GetHashCode(); }
        #endregion

        #region double
        public double Add(double x, double y) { return x + y; }
        public double Subtract(double x, double y) { return x - y; }
        public double Multiply(double x, double y) { return x * y; }
        public double Divide(double x, double y) { return x / y; }
        public double Modulus(double x, double y) { return x % y; }
        public double Negate(double x) { return -x; }
        double IArithmetic<double>.Zero { get { return 0; } }
        double IArithmetic<double>.Unit { get { return 1; } }
        double IArithmetic<double>.Value(int x) { return (double)x; }
        public bool IsZero(double x) { return x == 0; }
        public bool IsUnit(double x) { return x == 1; }
        public bool Equals(double x, double y) { return x == y; }
        public int GetHashCode(double x) { return x.GetHashCode(); }
        #endregion
    }
}