﻿#region Using directives

using System;
using System.Collections.Generic;

#endregion

namespace STL.Generic
{
    public delegate R NullaryFunction<R>();
    public delegate R UnaryFunction<A, R>(A arg);
    public delegate R BinaryFunction<A1, A2, R>(A1 arg1, A2 arg2);
    public delegate void BinaryFunction<A1, A2>(A1 arg1, A2 args);

    public static class Functional
    {
        public static BinaryFunction<T, T, T> Plus<T, I>() 
            where I : IArithmetic<T>
            where T : IComparable, IFormattable, IConvertible, IComparable<T>
        {
            I i = default(I);
            return i.Add;
        }

        public static BinaryFunction<T, T, T> Minus<T, I>() 
            where I : IArithmetic<T>
            where T : IComparable, IFormattable, IConvertible, IComparable<T>
        {
            I i = default(I);
            return i.Subtract;
        }

        public static BinaryFunction<T, T, T> Multiplies<T, I>() 
            where I : IArithmetic<T>
            where T : IComparable, IFormattable, IConvertible, IComparable<T>
        {
            I i = default(I);
            return i.Multiply;
        }

        public static BinaryFunction<T, T, T> Divides<T, I>() 
            where I : IArithmetic<T>
            where T : IComparable, IFormattable, IConvertible, IComparable<T>
        {
            I i = default(I);
            return i.Divide;
        }

        public static BinaryFunction<T, T, T> Modulus<T, I>() 
            where I : IArithmetic<T>
            where T : IComparable, IFormattable, IConvertible, IComparable<T>
        {
            I i = default(I);
            return i.Modulus;
        }

        public static BinaryFunction<T, T, bool> EqualTo<T>() 
            where T : IComparable, IFormattable, IConvertible, IComparable<T>
        {
            return delegate(T x, T y)
            {
                return ((IComparable<T>)x).CompareTo(y) == 0;
            };
        }

        public static BinaryFunction<T, T, bool> NotEqualTo<T>() 
            where T : IComparable, IFormattable, IConvertible, IComparable<T>
        {
            return delegate(T x, T y)
            {
                return ((IComparable<T>)x).CompareTo(y) != 0;
            };
        }

        public static BinaryFunction<T, T, bool> Greater<T>() 
            where T : IComparable, IFormattable, IConvertible, IComparable<T>
        {
            return delegate(T x, T y)
            {
                return ((IComparable<T>)x).CompareTo(y) > 0;
            };
        }

        public static BinaryFunction<T, T, bool> Less<T>() 
            where T : IComparable, IFormattable, IConvertible, IComparable<T>
        {
            return delegate(T x, T y)
            {
                return ((IComparable<T>)x).CompareTo(y) < 0;
            };
        }

        public static BinaryFunction<T, T, bool> GreaterEqual<T>() 
            where T : IComparable, IFormattable, IConvertible, IComparable<T>
        {
            return delegate(T x, T y)
            {
                return ((IComparable<T>)x).CompareTo(y) >= 0;
            };
        }

        public static BinaryFunction<T, T, bool> LessEqual<T>() 
            where T : IComparable, IFormattable, IConvertible, IComparable<T>
        {
            return delegate(T x, T y)
            {
                return ((IComparable<T>)x).CompareTo(y) <= 0;
            };
        }

        public static BinaryFunction<bool, bool, bool> LogicalAnd()
        {
            return delegate(bool x, bool y)
            {
                return x && y;
            };
        }

        public static BinaryFunction<bool, bool, bool> LogicalOr()
        {
            return delegate(bool x, bool y)
            {
                return x || y;
            };
        }

        public static UnaryFunction<bool, bool> LogicalNot()
        {
            return delegate(bool x)
            {
                return !x;
            };
        }

        public static UnaryFunction<T, bool> Not1<T>(UnaryFunction<T, bool> pr)
        {
            return delegate(T x)
            {
                return !pr(x);
            };
        }

        public static BinaryFunction<T, T, bool> Not2<T>(BinaryFunction<T, T, bool> pr)
        {
            return delegate(T x, T y)
            {
                return !pr(x, y);
            };
        }

        public static NullaryFunction<R>
            Bind<A, R>(UnaryFunction<A, R> pr, A arg)
        {
            return delegate()
            {
                return pr(arg);
            };
        }

        public static UnaryFunction<A2, R>
            Bind1st<A1, A2, R>(BinaryFunction<A1, A2, R> pr, A1 arg1)
        {
            return delegate(A2 arg2)
            {
                return pr(arg1, arg2);
            };
        }

        public static Action<A2>
            Bind1st<A1, A2>(BinaryFunction<A1, A2> pr, A1 arg1)
        {
            return delegate(A2 arg2)
            {
                pr(arg1, arg2);
            };
        }

        public static UnaryFunction<A1, R>
            Bind2nd<A1, A2, R>(BinaryFunction<A1, A2, R> pr, A2 arg2)
        {
            return delegate(A1 arg1)
            {
                return pr(arg1, arg2);
            };
        }

        public static Action<A1>
            Bind2nd<A1, A2>(BinaryFunction<A1, A2> pr, A2 arg2)
        {
            return delegate(A1 arg1)
            {
                pr(arg1, arg2);
            };
        }
    }
}
