﻿#region Using directives

using System;
using System.Collections.Generic;

#endregion

namespace STL.Generic
{
    public interface IForwardIterator<T> : ICloneable
    {
        T Value { get; set; }
        IForwardIterator<T> Next();
    }

    public interface IBidirectionalIterator<T> : IForwardIterator<T>
    {
        IBidirectionalIterator<T> Prev();
    }

    public interface IRandomIterator<T> : IBidirectionalIterator<T>
    {
        int Position { get; set; }
        IRandomIterator<T> Offset(int n);
    }

    public class ReverseIterator<T> : IBidirectionalIterator<T>
    {
        IRandomIterator<T> current;

        public ReverseIterator(IRandomIterator<T> x)
        {
            current = x;
        }

        public T Value
        {
            get
            {
                return current.Value;
            }
            set
            {
                current.Value = value;
            }
        }

        public IForwardIterator<T> Next()
        {
            current.Prev();
            return this;
        }

        public IBidirectionalIterator<T> Prev()
        {
            current.Next();
            return this;
        }

        public bool Equals(ReverseIterator<T> obj)
        {
            return current.Equals(obj.current);
        }

        public override bool Equals(object obj)
        {
            return Equals((ReverseIterator<T>)obj);
        }

        public override int GetHashCode()
        {
            return current.GetHashCode();
        }

        public object Clone()
        {
            return base.MemberwiseClone();
        }
    }
}
