﻿#region Using directives

using System;
using System.Collections.Generic;

#endregion

namespace STL.Generic
{
    public class Tuple
    {
        public object this[int index]
        {
            get
            {
                return GetType().GetField("_" + index).GetValue(this);
            }
            set
            {
                GetType().GetField("_" + index).SetValue(this, value);
            }
        }

        public int Length
        {
            get { return GetType().GetGenericArguments().Length; }
        }

        public static Tuple Make(params object[] args)
        {
            Type t = null;
            switch (args.Length)
            {
                case 1: t = typeof(Tuple<>);            break;
                case 2: t = typeof(Tuple<,>);           break;
                case 3: t = typeof(Tuple<,,>);          break;
                case 4: t = typeof(Tuple<,,,>);         break;
                case 5: t = typeof(Tuple<,,,,>);        break;
                case 6: t = typeof(Tuple<,,,,,>);       break;
                case 7: t = typeof(Tuple<,,,,,,>);      break;
                case 8: t = typeof(Tuple<,,,,,,,>);     break;
                case 9: t = typeof(Tuple<,,,,,,,,>);    break;
                case 10:t = typeof(Tuple<,,,,,,,,,>);   break;
                default:throw new ArgumentException();
            }
            Type[] types = Type.GetTypeArray(args);
            t = t.BindGenericParameters(types);
            return Activator.CreateInstance(t, args) as Tuple;
        }

        public static T Make<T>(params object[] args) where T : Tuple
        {
            return Make(args) as T;
        }
    }

    public class Tuple<T0> : Tuple
    {
        public T0 _0;

        public Tuple(T0 v0)
        {
            _0 = v0;
        }

        public void Tie(ref T0 v0)
        {
            v0 = _0;
        }
    }

    public class Tuple<T0, T1> : Tuple
    {
        public T0 _0; public T1 _1;

        public Tuple(T0 v0, T1 v1) 
        {
            _0 = v0; _1 = v1;
        }

        public void Tie(ref T0 v0, ref T1 v1)
        {
            v0 = _0; v1 = _1;
        }
    }

    public class Tuple<T0, T1, T2> : Tuple
    {
        public T0 _0; public T1 _1; public T2 _2;

        public Tuple(T0 v0, T1 v1, T2 v2) 
        {
            _0 = v0; _1 = v1; _2 = v2;
        }

        public void Tie(ref T0 v0, ref T1 v1, ref T2 v2)
        {
            v0 = _0; v1 = _1; v2 = _2;
        }
    }

    public class Tuple<T0, T1, T2, T3> : Tuple
    {
        public T0 _0; public T1 _1; public T2 _2; public T3 _3;

        public Tuple(T0 v0, T1 v1, T2 v2, T3 v3) 
        {
            _0 = v0; _1 = v1; _2 = v2; _3 = v3;
        }

        public void Tie(ref T0 v0, ref T1 v1, ref T2 v2, ref T3 v3)
        {
            v0 = _0; v1 = _1; v2 = _2; v3 = _3;
        }
    }

    public class Tuple<T0, T1, T2, T3, T4> : Tuple
    {
        public T0 _0; public T1 _1; public T2 _2; public T3 _3; public T4 _4;

        public Tuple(T0 v0, T1 v1, T2 v2, T3 v3, T4 v4) 
        {
            _0 = v0; _1 = v1; _2 = v2; _3 = v3; _4 = v4;
        }

        public void Tie(ref T0 v0, ref T1 v1, ref T2 v2, ref T3 v3, ref T4 v4)
        {
            v0 = _0; v1 = _1; v2 = _2; v3 = _3; v4 = _4;
        }
    }

    public class Tuple<T0, T1, T2, T3, T4, T5> : Tuple
    {
        public T0 _0; public T1 _1; public T2 _2; public T3 _3; public T4 _4;
        public T5 _5;

        public Tuple(T0 v0, T1 v1, T2 v2, T3 v3, T4 v4, T5 v5) 
        {
            _0 = v0; _1 = v1; _2 = v2; _3 = v3; _4 = v4;
            _5 = v5;
        }

        public void Tie(ref T0 v0, ref T1 v1, ref T2 v2, ref T3 v3, ref T4 v4, 
                        ref T5 v5)
        {
            v0 = _0; v1 = _1; v2 = _2; v3 = _3; v4 = _4;
            v5 = _5;
        }
    }

    public class Tuple<T0, T1, T2, T3, T4, T5, T6> : Tuple
    {
        public T0 _0; public T1 _1; public T2 _2; public T3 _3; public T4 _4;
        public T5 _5; public T6 _6;

        public Tuple(T0 v0, T1 v1, T2 v2, T3 v3, T4 v4, T5 v5, T6 v6) 
        {
            _0 = v0; _1 = v1; _2 = v2; _3 = v3; _4 = v4; 
            _5 = v5; _6 = v6;
        }

        public void Tie(ref T0 v0, ref T1 v1, ref T2 v2, ref T3 v3, ref T4 v4, 
                        ref T5 v5, ref T6 v6)
        {
            v0 = _0; v1 = _1; v2 = _2; v3 = _3; v4 = _4;
            v5 = _5; v6 = _6;
        }
    }

    public class Tuple<T0, T1, T2, T3, T4, T5, T6, T7> : Tuple
    {
        public T0 _0; public T1 _1; public T2 _2; public T3 _3; public T4 _4;
        public T5 _5; public T6 _6; public T7 _7;

        public Tuple(T0 v0, T1 v1, T2 v2, T3 v3, T4 v4, T5 v5, T6 v6, T7 v7) 
        {
            _0 = v0; _1 = v1; _2 = v2; _3 = v3; _4 = v4;
            _5 = v5; _6 = v6; _7 = v7;
        }

        public void Tie(ref T0 v0, ref T1 v1, ref T2 v2, ref T3 v3, ref T4 v4,
                        ref T5 v5, ref T6 v6, ref T7 v7)
        {
            v0 = _0; v1 = _1; v2 = _2; v3 = _3; v4 = _4;
            v5 = _5; v6 = _6; v7 = _7;
        }
    }

    public class Tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8> : Tuple
    {
        public T0 _0; public T1 _1; public T2 _2; public T3 _3; public T4 _4;
        public T5 _5; public T6 _6; public T7 _7; public T8 _8;

        public Tuple(T0 v0, T1 v1, T2 v2, T3 v3, T4 v4, T5 v5, T6 v6, T7 v7, T8 v8) 
        {
            _0 = v0; _1 = v1; _2 = v2; _3 = v3; _4 = v4;
            _5 = v5; _6 = v6; _7 = v7; _8 = v8;
        }

        public void Tie(ref T0 v0, ref T1 v1, ref T2 v2, ref T3 v3, ref T4 v4,
                        ref T5 v5, ref T6 v6, ref T7 v7, ref T8 v8)
        {
            v0 = _0; v1 = _1; v2 = _2; v3 = _3; v4 = _4;
            v5 = _5; v6 = _6; v7 = _7; v8 = _8;
        }
    }

    public class Tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9> : Tuple
    {
        public T0 _0; public T1 _1; public T2 _2; public T3 _3; public T4 _4;
        public T5 _5; public T6 _6; public T7 _7; public T8 _8; public T9 _9;

        public Tuple(T0 v0, T1 v1, T2 v2, T3 v3, T4 v4, T5 v5, T6 v6, T7 v7, T8 v8, T9 v9) 
        {
            _0 = v0; _1 = v1; _2 = v2; _3 = v3; _4 = v4;
            _5 = v5; _6 = v6; _7 = v7; _8 = v8; _9 = v9;
        }

        public void Tie(ref T0 v0, ref T1 v1, ref T2 v2, ref T3 v3, ref T4 v4,
                        ref T5 v5, ref T6 v6, ref T7 v7, ref T8 v8, ref T9 v9)
        {
            v0 = _0; v1 = _1; v2 = _2; v3 = _3; v4 = _4;
            v5 = _5; v6 = _6; v7 = _7; v8 = _8; v9 = _9;
        }
    }
}